# (c) 2004-2005, Red Hat, Inc
# All rights reserved
#

package RHN::SessionSwap;
use strict;

use Digest::MD5;
use Digest::HMAC_SHA1 qw/hmac_sha1_hex/;

use PXT::Config;

# this is basically a copy of RHN::Session's key stuff
sub generate_swap_key {
  my $class = shift;
  my $id = shift;

  my $chaff = join(":",
		   PXT::Config->get('session_swap_secret_1'),
		   PXT::Config->get('session_swap_secret_2'),
		   $id,
		   PXT::Config->get('session_swap_secret_3'),
		   PXT::Config->get('session_swap_secret_4')
		  );

  my $ret = Digest::MD5::md5_hex($chaff);

  return $ret;
}

sub encode_data {
  my $class = shift;
  my @in = @_;

  die "Invalid data to encode: @in"
    if grep { $_ =~ /[^0-9a-f]/ } @in;

  my $in = join(":", @in);

  return join("x", $in, $class->generate_swap_key($in));
}

sub extract_data {
  my $class = shift;
  my $in = shift;

  die "extract_data called on undefined value" unless defined $in;

  my ($data) = split /x/, $in, 2;
  my @atoms = split /:/, $data;

  if ($in eq $class->encode_data(@atoms)) {
    return @atoms;
  }
  else {
    die "Data tampered with: $in does not pass checksum test";
  }
}

sub rhn_hmac_data {
  my $class = shift;
  my $hmac_data = join("\0", @_);

  my $hmac_key = join("",
		      PXT::Config->get('session_swap_secret_4'),
		      PXT::Config->get('session_swap_secret_3'),
		      PXT::Config->get('session_swap_secret_2'),
		      PXT::Config->get('session_swap_secret_1')
		  );
  return hmac_sha1_hex($hmac_data, $hmac_key);
}

1;
